@@ -36,6 +36,7 @@ public:
   virtual ~ProxyServerMediaSubsession();
 
   char const* codecName() const { return fCodecName; }
+  char const* url() const { return ((ProxyServerMediaSession*)fParentSession)->url(); }
 
 private: // redefined virtual functions
   virtual FramedSource* createNewStreamSource(unsigned clientSessionId,
@@ -66,7 +67,7 @@ private:
 ////////// ProxyServerMediaSession implementation //////////
 
 UsageEnvironment& operator<<(UsageEnvironment& env, const ProxyServerMediaSession& psms) { // used for debugging
-  return env << "ProxyServerMediaSession[\"" << psms.url() << "\"]";
+  return env << "ProxyServerMediaSession[" << psms.url() << "]";
 }
 
 ProxyRTSPClient*
@@ -238,7 +239,7 @@ static void continueAfterGET_PARAMETER(RTSPClient* rtspClient, int resultCode, c
 ////////// "ProxyRTSPClient" implementation /////////
 
 UsageEnvironment& operator<<(UsageEnvironment& env, const ProxyRTSPClient& proxyRTSPClient) { // used for debugging
-  return env << "ProxyRTSPClient[\"" << proxyRTSPClient.url() << "\"]";
+  return env << "ProxyRTSPClient[" << proxyRTSPClient.url() << "]";
 }
 
 ProxyRTSPClient::ProxyRTSPClient(ProxyServerMediaSession& ourServerMediaSession, char const* rtspURL,
@@ -330,7 +331,7 @@ void ProxyRTSPClient::continueAfterLivenessCommand(int resultCode, Boolean serve
   scheduleLivenessCommand();
 }
 
-#define SUBSESSION_TIMEOUT_SECONDS 10 // how many seconds to wait for the last track's "SETUP" to be done (note below)
+#define SUBSESSION_TIMEOUT_SECONDS 5 // how many seconds to wait for the last track's "SETUP" to be done (note below)
 
 void ProxyRTSPClient::continueAfterSETUP(int resultCode) {
   if (resultCode != 0) {
@@ -347,7 +348,8 @@ void ProxyRTSPClient::continueAfterSETUP(int resultCode) {
 	    << "; numSubsessions " << fSetupQueueHead->fParentSession->numSubsessions() << "\n\tqueue:";
     for (ProxyServerMediaSubsession* p = fSetupQueueHead; p != NULL; p = p->fNext) {
       envir() << "\t" << p->codecName();
-      if (p->fNext == fSetupQueueHead || p->fNext == p) { fprintf(stderr, "##### INTERNAL ERROR 1\n"); break; } //##### TEMP FOR DEBUGGING
+      if (p->fNext == fSetupQueueHead) { fprintf(stderr, "##### INTERNAL ERROR 1.1\n"); break; } //##### TEMP FOR DEBUGGING
+      else if (p->fNext == p) { fprintf(stderr, "##### INTERNAL ERROR 1.2\n"); break; } //##### TEMP FOR DEBUGGING
     }
     envir() << "\n";
   }
@@ -479,7 +481,7 @@ ProxyServerMediaSubsession
 }
 
 UsageEnvironment& operator<<(UsageEnvironment& env, const ProxyServerMediaSubsession& psmss) { // used for debugging
-  return env << "ProxyServerMediaSubsession[\"" << psmss.codecName() << "\"]";
+  return env << "ProxyServerMediaSubsession[" << psmss.url() << "," << psmss.codecName() << "]";
 }
 
 ProxyServerMediaSubsession::~ProxyServerMediaSubsession() {
@@ -565,11 +567,17 @@ FramedSource* ProxyServerMediaSubsession::createNewStreamSource(unsigned clientS
       if (queueWasEmpty) {
 	if (proxyRTSPClient->fSetupQueueTail != NULL) fprintf(stderr, "##### INTERNAL ERROR 3\n");
 	proxyRTSPClient->fSetupQueueHead = this;
+	proxyRTSPClient->fSetupQueueTail = this;
       } else {
-	if (proxyRTSPClient->fSetupQueueTail == NULL) fprintf(stderr, "##### INTERNAL ERROR 4\n"); else //##### TEMP FOR DEBUGGING
-	proxyRTSPClient->fSetupQueueTail->fNext = this;
+	if (proxyRTSPClient->fSetupQueueTail == NULL) fprintf(stderr, "##### INTERNAL ERROR 4\n"); else { //##### TEMP FOR DEBUGGING
+	  Boolean err5 = False; //##### TEMP FOR DEBUGGING
+	  for (ProxyServerMediaSubsession* psms = proxyRTSPClient->fSetupQueueHead; psms != NULL; psms = psms->fNext) { if (psms == this) { err5 = True; break; } } //##### TEMP FOR DEBUGGING
+	  if (err5) fprintf(stderr, "##### INTERNAL ERROR 5\n"); else { //##### TEMP FOR DEBUGGING
+	    proxyRTSPClient->fSetupQueueTail->fNext = this;
+	    proxyRTSPClient->fSetupQueueTail = this;
+	  } //##### TEMP FOR DEBUGGING
+	} //##### TEMP FOR DEBUGGING
       }
-      proxyRTSPClient->fSetupQueueTail = this;
 
       // Hack: If there's already a pending "SETUP" request (for another track), don't send this track's "SETUP" right away, because
       // the server might not properly handle 'pipelined' requests.  Instead, wait until after previous "SETUP" responses come back.
@@ -608,8 +616,16 @@ void ProxyServerMediaSubsession::closeStreamSource(FramedSource* inputSource) {
     ProxyServerMediaSession* const sms = (ProxyServerMediaSession*)fParentSession;
     ProxyRTSPClient* const proxyRTSPClient = sms->fProxyRTSPClient;
     if (proxyRTSPClient->fLastCommandWasPLAY) { // so that we send only one "PAUSE"; not one for each subsession
-      proxyRTSPClient->sendPauseCommand(fClientMediaSubsession.parentSession(), NULL, proxyRTSPClient->auth());
-      proxyRTSPClient->fLastCommandWasPLAY = False;
+      if (fParentSession->referenceCount() > 1) {
+	// There are other client(s) still streaming other subsessions of this stream.
+	// Therefore, we don't send a "PAUSE" for the whole stream, but only for the sub-stream:
+	proxyRTSPClient->sendPauseCommand(fClientMediaSubsession, NULL, proxyRTSPClient->auth());
+      } else {
+	// Normal case: There are no other client still streaming (parts of) this stream.
+	// Send a "PAUSE" for the whole stream.
+	proxyRTSPClient->sendPauseCommand(fClientMediaSubsession.parentSession(), NULL, proxyRTSPClient->auth());
+	proxyRTSPClient->fLastCommandWasPLAY = False;
+      }
     }
   }
 }
